package catnet.negotiation;

import java.util.ArrayList;
import catnet.util.Tool;
import catnet.negotiation.ProposalLineItem;

/**
 *  Proposals to be sent between the agents over the Lars server. A proposal is
 *  in its simplest form a list of ProposalLineItems. Additional information is
 *  the documentType (whether the list of goods is to be sold (BID) or bought
 *  (ASK).
 *
 *@author     Stefan Avram <a href="mailto:avram@informatik.uni-freiburg.de">
 *      avram@informatik.uni-freiburg.de</a> <BR>
 *      Stefan Mutter <a href="mailto:mutter@informatik.uni-freiburg.de">
 *      mutter@informatik.uni-freiburg.de</a>
 *@created    25. Oktober 2001
 *@version    1.0, 01/06/25
 *@see        #ProposalLineItem
 *@note:      this class has a natural ordering that is inconsistent with
 *      equals.
 */
public class Proposal {

    /**
     *  Description of the Field
     */
    private java.util.ArrayList lineItems = new ArrayList();

    /**
     *  Description of the Field
     */
    public final static int ASK = 1;
    /**
     *  Description of the Field
     */
    public final static int BID = 0;

    /**
     *  Description of the Field
     */
    public final static String[] typeText = {"ASK", "BID"};

    /**
     *  Description of the Field
     */
    private int documentType;

    /**
     *  Description of the Field
     */
    private String owner = "";


    /**
     *  Constructor for the Proposal object
     *
     *@param  type  Description of the Parameter
     *@param  l     Description of the Parameter
     */
    public Proposal(int type, ArrayList l) {
        this.documentType = type;
        this.lineItems = l;
    }


    /**
     *  Constructor for the Proposal object
     *
     *@param  type  Description of the Parameter
     *@param  p     Description of the Parameter
     */
    public Proposal(int type, ProposalLineItem p) {
        this.documentType = type;
        this.lineItems.add(0, p);
    }


    /**
     *  Constructor for the Proposal object
     *
     *@param  type  Description of the Parameter
     *@param  id    Description of the Parameter
     *@param  q     Description of the Parameter
     *@param  d     Description of the Parameter
     */
    public Proposal(int type, Integer id, int q, double d) {
        this.documentType = type;
        this.lineItems.add(0, new ProposalLineItem(id, q, d));
    }


    /**
     *  Adds a feature to the LineItem attribute of the Proposal object
     *
     *@param  p  The feature to be added to the LineItem attribute
     */
    public void addLineItem(ProposalLineItem p) {
        this.lineItems.add(p);
    }


    /**
     *  Gets the lineItem attribute of the Proposal object
     *
     *@param  index  Description of the Parameter
     *@return        The lineItem value
     */
    public ProposalLineItem getLineItem(int index) {
        return (ProposalLineItem) this.lineItems.get(index);
    }


    /**
     *  Gets the lineItem attribute of the Proposal object
     *
     *@return    The lineItem value
     */
    public ProposalLineItem getLineItem() {
        return (ProposalLineItem) this.lineItems.get(0);
    }


    /**
     *  Gets the money attribute of the Proposal object
     *
     *@return    The money value
     *@todo      compute the sum of all proposallineitems
     */
    public double getUnitPrice() {
        return ((ProposalLineItem) this.lineItems.get(0)).getUnitPrice();
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */

    /**
     *  Gets the factorID attribute of the Proposal object
     *
     *@return    The factorID value
     */
    public Integer getFactorID() {
        return ((ProposalLineItem) this.lineItems.get(0)).getFactorID();
    }


    /**
     *  Gets the quantity attribute of the Proposal object
     *
     *@return    The quantity value
     */
    public int getQuantity() {
        return ((ProposalLineItem) this.lineItems.get(0)).getProductQuantity();
    }


    /**
     *  Sets the quantity attribute of the Proposal object
     *
     *@param  q  The new quantity value
     */
    public void setQuantity(int q) {
        ProposalLineItem pli = (ProposalLineItem) lineItems.get(0);
        pli.setProductQuantity(q);
        lineItems.add(0, pli);
    }



    /**
     *  Gives back the documentType of the Proposal. <BR>
     *  BID=0 <BR>
     *  OFFER=1
     *
     *@return    the documentType of the Proposal.
     */
    public int getDocumentType() {
        return documentType;
    }


    /**
     *  Gets the type attribute of the Proposal object
     *
     *@return    The type value
     */
    public int getType() {
        return getDocumentType();
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public int switchDocumentType() {
//        return (this.getDocumentType() == Proposal.BID ? Proposal.ASK : Proposal.BID);
        if (this.getDocumentType() == Proposal.BID) {
            return Proposal.ASK;
        } else {
            return Proposal.BID;
        }
    }


    /**
     *  Description of the Method
     *
     *@param  i  Description of the Parameter
     *@return    Description of the Return Value
     */
    public final static int switchDocumentType(int i) {
        if (i == Proposal.BID) {
            return Proposal.ASK;
        } else {
            return Proposal.BID;
        }
    }


    /**
     *  Clones the actual proposal with new the other documentType.
     *
     *@return    the reversed cloned proposal.
     */
    public Proposal reverseProposal() {
        Proposal p = new Proposal(switchDocumentType(), this.lineItems);
        return p;
    }


    /**
     *  Clones the actual proposal with new price and switches the documenttype
     *  from BID to ASK and vice versa.
     *
     *@param  d  Description of the Parameter
     *@return    the cloned proposal.
     */
    public Proposal replyProposal(double d) {
        Proposal p = new Proposal(this.switchDocumentType(), this.getFactorID(), this.getQuantity(), d);
        return p;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public String toString() {
        String logLine = "";
        int j = this.getFactorID().intValue();

        logLine = this.getDocumentType() + "\t" + j + "\t" + this.getUnitPrice() + "\t" + this.getQuantity();

        for (int i = 1; i < 5; i++) {
            if (j != i) {
                logLine += ("\t");
            } else {
                logLine += ("\t" + this.getUnitPrice());
            }
        }
        return logLine;
    }


    /**
     *  Gets the owner attribute of the Proposal object
     *
     *@return    The owner value
     */
    public String getOwner() {
        return owner;
    }


    /**
     *  Sets the owner attribute of the Proposal object
     *
     *@param  owner  The new owner value
     */
    public void setOwner(String owner) {
        this.owner = owner;
    }
}
